import unittest

from pySRU.tests.MagneticFieldTest      import MagneticFieldTest
from pySRU.tests.MagneticStructureTest  import MagneticStructureTest
from pySRU.tests.MainTest               import MainTest
from pySRU.tests.RadiationFactorytest   import RadiationFactoryTest
from pySRU.tests.RadiationTest          import RadiationTest
from pySRU.tests.SimulationTest         import UndulatorSimulationTest
from pySRU.tests.SourceTest             import UndulatorParameterTest
from pySRU.tests.TrajectoryFactoryTest  import TrajectoryFactoryTest
from pySRU.tests.TrajectoryTest         import TrajectoryTest

def suite():
    """
    Gathers all the tests in a test suite.
    """
    suites = (
        unittest.makeSuite(MagneticFieldTest, 'test'),
        unittest.makeSuite(MagneticStructureTest, 'test'),
        unittest.makeSuite(MainTest, 'test'),
        unittest.makeSuite(RadiationFactoryTest, 'test'),
        unittest.makeSuite(RadiationTest, 'test'),
        unittest.makeSuite(UndulatorSimulationTest, 'test'),
        unittest.makeSuite(UndulatorParameterTest, 'test'),
        unittest.makeSuite(TrajectoryFactoryTest, 'test'),
        unittest.makeSuite(TrajectoryTest, 'test'),

    )
    return unittest.TestSuite(suites)


if __name__ == "__main__":

    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import unittest
import numpy as np
import scipy.constants as codata

from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator
from pySRU.ElectronBeam import ElectronBeam
from pySRU.Source import Source
from pySRU.TrajectoryFactory import TrajectoryFactory,TRAJECTORY_METHOD_ANALYTIC,    TRAJECTORY_METHOD_ODE
from pySRU.RadiationFactory import RadiationFactory , RADIATION_METHOD_APPROX_FARFIELD,RADIATION_METHOD_NEAR_FIELD
from pySRU.Simulation import Simulation,create_simulation

class UndulatorSimulationTest(unittest.TestCase):


    def test_simulation(self):
        electron_beam_test = ElectronBeam(Electron_energy=1.3, I_current=1.0)
        beam_ESRF = ElectronBeam(Electron_energy=6.0, I_current=0.2)
        undulator_test = Undulator(K=1.87, period_length=0.035, length=0.035 * 14)
        ESRF18 = Undulator(K=1.68, period_length=0.018, length=2.0)

        sim_test = create_simulation(magnetic_structure=undulator_test,electron_beam=electron_beam_test,
                                     traj_method=TRAJECTORY_METHOD_ANALYTIC,rad_method=RADIATION_METHOD_NEAR_FIELD)
        self.assertFalse(sim_test.radiation.distance == None)
        source_test=sim_test.source

        self.assertFalse(all(sim_test.trajectory_fact.initial_condition==
                             source_test.choose_initial_contidion_automatic()))
        ref=sim_test.copy()
        rad_max = sim_test.radiation.max()

        # test change
        sim_test.change_radiation_method(RADIATION_METHOD_APPROX_FARFIELD)
        self.assertEqual(sim_test.radiation_fact.method, RADIATION_METHOD_APPROX_FARFIELD)
        self.assertFalse(ref.radiation_fact.method==sim_test.radiation_fact.method)
        self.assertFalse(np.all(ref.radiation.intensity == sim_test.radiation.intensity))
        self.assertAlmostEqual(ref.radiation.intensity[0][0]/rad_max, sim_test.radiation.intensity[0][0]/rad_max, 3)

        sim_test.change_trajectory_method(TRAJECTORY_METHOD_ODE)
        self.assertEqual(sim_test.trajectory_fact.method, TRAJECTORY_METHOD_ODE)
        self.assertFalse(ref.trajectory_fact.method==sim_test.trajectory_fact.method)
        time_diff=np.abs(ref.trajectory.t - sim_test.trajectory.t)
        self.assertTrue(np.all(time_diff<=1e-19))
        self.assertFalse(np.all(ref.trajectory.x == sim_test.trajectory.x))
        self.assertFalse(np.all(ref.radiation.intensity == sim_test.radiation.intensity))
        rad_max = sim_test.radiation.max()
        self.assertAlmostEqual(ref.radiation.intensity[0][0]/rad_max, sim_test.radiation.intensity[0][0]/rad_max, 1)

        sim_test.change_Nb_pts_trajectory(ref.trajectory_fact.Nb_pts+1)
        self.assertEqual(sim_test.trajectory_fact.Nb_pts,ref.trajectory_fact.Nb_pts+1)
        self.assertEqual(sim_test.trajectory.nb_points(), ref.trajectory_fact.Nb_pts+1)
        self.assertFalse(ref.trajectory_fact.Nb_pts == sim_test.trajectory_fact.Nb_pts)
        self.assertAlmostEqual(ref.radiation.intensity[0][0]/rad_max,sim_test.radiation.intensity[0][0]/rad_max,1)

        sim_test.change_Nb_pts_radiation(100)
        self.assertEqual(sim_test.radiation_fact.Nb_pts,100)
        self.assertFalse(np.all(ref.radiation.X == sim_test.radiation.X))
        self.assertTrue(ref.radiation.X.min() == sim_test.radiation.X.min())
        self.assertTrue(ref.radiation.X.max() == sim_test.radiation.X.max())
        self.assertTrue(ref.radiation.Y.min() == sim_test.radiation.Y.min())
        self.assertTrue(ref.radiation.Y.max() == sim_test.radiation.Y.max())
        self.assertFalse(len(ref.radiation.X) == len(sim_test.radiation.X))

        sim_test.change_distance(50)
        self.assertEqual(sim_test.radiation.distance,50)
        self.assertFalse(ref.radiation.distance == sim_test.radiation.distance)

        sim_test.change_photon_frequency(source_test.harmonic_frequency(1) * 0.8)
        self.assertEqual(sim_test.radiation_fact.photon_frequency,source_test.harmonic_frequency(1)*0.8)
        self.assertFalse(ref.radiation_fact.photon_frequency == sim_test.radiation_fact.photon_frequency)


        # nb_pts=np.arange(500,2001,500,dtype='int')
        # err=sim_test.error_radiation_method_nb_pts_traj(RADIATION_METHOD_APPROX_FARFIELD,nb_pts=nb_pts)
        # self.assertLessEqual((err.max() / rad_max), 1e-2, 1)
        #
        # distance=np.arange(20,101,20,dtype='int')
        # err = sim_test.error_radiation_method_distance(RADIATION_METHOD_APPROX_FARFIELD, D=distance)
        # self.assertLessEqual(err.max()/rad_max, 1e-2, 1)




# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import numpy as np
import scipy.constants as codata
from mpl_toolkits.mplot3d import Axes3D
import scipy.integrate as integrate
import scipy.interpolate as interpolate
import time
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator
from pySRU.MagneticStructureBendingMagnet import MagneticStructureBendingMagnet  as BM
from pySRU.ElectronBeam import ElectronBeam
from pySRU.Source import Source
from pySRU.Simulation import Simulation ,create_simulation
from pySRU.TrajectoryFactory import TRAJECTORY_METHOD_ANALYTIC,TRAJECTORY_METHOD_ODE
from pySRU.RadiationFactory import RADIATION_METHOD_NEAR_FIELD, RADIATION_METHOD_APPROX_FARFIELD,RADIATION_METHOD_APPROX


eV_to_J=1.602176487e-19
######################################################

#BM_test=MagneticStructureBendingMagnet(Bo=0.8, div=5e-3, R=25.0)

E_1=7876.0

beam_test=ElectronBeam(Electron_energy=1.3, I_current=1.0)
beam_ESRF=ElectronBeam(Electron_energy=6.0, I_current=0.2)
und_test=Undulator(  K = 1.87,  period_length= 0.035, length=0.035 * 14)
ESRF18=Undulator( K = 1.68, period_length = 0.018, length=2.0)
ESRFBM=BM(Bo=0.8,horizontale_divergeance=0.005,electron_energy=6.0)



vx= 2e-4
vz= np.sqrt(beam_test.electron_speed()**2-vx**2)*codata.c

# initial_cond=np.array([ vx*codata.c,  0.00000000e+00 ,vz , 0.0 , 0.0 ,-0.42,])
#X=np.linspace(-0.02,0.02,150)
#Y=np.linspace(-0.02,0.02,150)
sim_test = create_simulation(magnetic_structure=und_test, electron_beam=beam_test, traj_method=TRAJECTORY_METHOD_ANALYTIC,
                            rad_method=RADIATION_METHOD_APPROX_FARFIELD, Nb_pts_trajectory=1000,distance=100)
sim_test.calculate_on_central_cone()
sim_test.change_energy_eV(E=2000)
#sim_test.change_harmonic_number(5.5)
#print(sim_test.source.choose_nb_pts_trajectory())
#print(sim_test.source.choose_distance_automatic())
#sim_test.print_parameters()


#sim_test.trajectory.plot_3D(title="Analytical electron trajectory in bending magnet")
#sim_test.trajectory.plot(title="Analytical electron trajectory in undulator")
#sim_test.calculate_on_central_cone()
#sim_test.calculate_spectrum_on_axis()
# omega1=sim_test.source.harmonic_frequency(1)
# omega_array=np.arange(.9*omega1,3.06*omega1,0.01*omega1)
# sim_test.calculate_spectrum_central_cone(abscissas_array=omega_array)
# sim_test.change_harmonic_number(3.)
#print (sim_test.radiation.max())
# sim_test.radiation.plot(title="radiation intensity from ODE trajectory solution ")
sim_test.radiation.plot(title="radiation intensity , approx ff 1 formula")



def erreur_near_ff(und,beam):
    sim_test_analy = create_simulation(magnetic_structure=und, electron_beam=beam,
                                       traj_method=TRAJECTORY_METHOD_ANALYTIC,Nb_pts_radiation=31,
                                       rad_method=RADIATION_METHOD_NEAR_FIELD,
                                       Nb_pts_trajectory=5000,distance=100)
    #sim_test_analy.change_harmonic_number(5.5)
    #sim_test_analy.calculate_on_central_cone()
    rad_max=sim_test_analy.radiation.max()
    print(rad_max)
    rad_max_theo=sim_test_analy.source.theoretical_flux_on_axis(frequency=sim_test_analy.radiation_fact.photon_frequency)
    print(rad_max_theo)
    d = np.linspace(20,60, 21)
    error_rad=sim_test_analy.error_radiation_method_distance(method=RADIATION_METHOD_APPROX,D=d)

    import matplotlib.pyplot as plt
    plt.plot(d,error_rad)
    plt.xlabel("distance")
    plt.ylabel("error")
    plt.title('absolut error between two radiation method')
    plt.show()
    plt.plot(d,error_rad/rad_max)
    plt.xlabel("distance")
    plt.ylabel("error")
    plt.title('relative error between two radiation method')
    plt.show()

#erreur_near_ff(ESRF18,beam_ESRF)
#erreur_near_ff(und_test,beam_test)

def erreur_nb_pts_traj(und,beam):
    sim_test_analy = create_simulation(magnetic_structure=und, electron_beam=beam,
                                       traj_method=TRAJECTORY_METHOD_ANALYTIC,Nb_pts_radiation=50,
                                       rad_method=RADIATION_METHOD_NEAR_FIELD,
                                       Nb_pts_trajectory=20000,distance=20)
    rad_ref=sim_test_analy.radiation.intensity
    rad_max=sim_test_analy.radiation.max()
    print(rad_max)
    rad_max_theo=sim_test_analy.source.theoretical_flux_on_axis(frequency=sim_test_analy.radiation_fact.photon_frequency)
    # print(rad_max_theo)
    # print((rad_max-rad_max_theo))
    np_pts= np.arange(300,500,10)
    error_rad=sim_test_analy.error_radiation_nb_pts_traj(good_value=rad_ref,nb_pts=np_pts)


    import matplotlib.pyplot as plt
    plt.plot(np_pts,error_rad)
    plt.xlabel("number of point")
    plt.ylabel("error")
    plt.title('absolut error, near field formula')
    plt.show()
    plt.plot(np_pts,error_rad/rad_max)
    plt.xlabel("number of point")
    plt.ylabel("error")
    plt.title('relative error, near field formula')
    plt.show()

#erreur_nb_pts_traj(und_test,beam_test)




# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import numpy as np
import scipy.constants as codata
from scipy.special import jn,yn,jv,yv
from pySRU.MagneticField import MagneticField
from pySRU.ElectronBeam import ElectronBeam
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane
from pySRU.Source import Source,PLANE_UNDULATOR,BENDING_MAGNET



class SourceUndulatorPlane(Source):
    def __init__(self, electron_beam, undulator, magnetic_field=None):
        super(self.__class__, self).__init__(electron_beam=electron_beam,
                                             magnetic_field=magnetic_field,
                                             magnetic_structure=undulator)



    def copy(self):
        return SourceUndulatorPlane(electron_beam=self.electron_beam.copy(),
                                    magnetic_field=self.magnetic_field.copy(),
                                    undulator=self.magnetic_structure.copy())

    def period_number(self):
        return self.magnetic_structure.period_number()

    def magnetic_field_strength(self):
        return self.magnetic_structure.magnetic_field_strength()

    ###############
    # choix automatic lors de la construction d'une simulation
    ################

    def choose_distance_automatic(self, alpha,photon_frequency=None):
        lim = self.magnetic_structure.length
        return lim * 10 ** alpha

    # def choose_distance_automatic(self, alpha=0.001,photon_frequency=None):
    #     if photon_frequency == None:
    #         photon_frequency = self.harmonic_frequency(1)
    #     cste=(photon_frequency/codata.c)
    #     cste2=(self.magnetic_structure.length**2)/(4.*alpha)
    #     return cste*cste2

    def choose_nb_pts_trajectory(self, alpha=0.01,photon_frequency=None):
        if photon_frequency==None :
            photon_frequency=self.harmonic_frequency(1)
        K=self.magnetic_structure.K
        #cste=K/(12*np.sqrt(photon_frequency)*np.sqrt(self.Fn(n=1)))
        cste = K / (12  * np.sqrt(self.Fn(n=1)))
        cste2=2.*np.pi*self.magnetic_structure.length*photon_frequency/(codata.c*self.Lorentz_factor())
        cste3=np.sqrt(cste/alpha)
        cst4=(cste2*cste3)/np.sqrt(np.sqrt(photon_frequency))
        return cst4*2.

    def choose_initial_contidion_automatic(self):
        Zo=-self.magnetic_structure.length/2.-5.*self.magnetic_structure.period_length
        ic=np.array([0.0,0.0,self.electron_speed()*codata.c,0.0,0.0,Zo])
        return ic

    def choose_photon_frequency(self):
        first_harm=self.harmonic_frequency(harmonic_number=1)
        return first_harm

    #se demander quel angles choisir automatiquement
    def choose_angle_deflection_max(self):
        gamma=self.Lorentz_factor()
        return self.magnetic_structure.K/gamma


    ###########
    #property of the trajectory
    ##################

    def average_z_speed_in_undulator(self):
        Beta_et = 1.0 -( 1.+(self.magnetic_structure.K ** 2) / 2.0) / (2.0 * self.Lorentz_factor() ** 2)
        return Beta_et

    def analytical_times_vector(self, Nb_pts):
        to = self.magnetic_structure.length / (2. * self.average_z_speed_in_undulator() * codata.c)
        time = np.linspace(-to, to, Nb_pts)
        return time

    def construct_times_vector(self, initial_contition, Nb_pts):
        to = initial_contition[5] / (self.average_z_speed_in_undulator() * codata.c)
        if to < 0.0:
            time = np.linspace(to, -to, Nb_pts)
        elif to == 0.0:
            t1 = self.magnetic_structure.length / 2.0 + 5. * self.magnetic_structure.period_length
            time = np.linspace(to, t1)
        else:
            delta_t = to - self.magnetic_structure.length / (2. * self.average_z_speed_in_undulator() * codata.c)
            if delta_t > 0.0:
                time = np.linspace(to, to + 2. * delta_t, Nb_pts)
            else:
                time = np.linspace(to, 2. * to, Nb_pts)
        return time

    def atol_for_ODE_method(self):
        atol_vx=self.magnetic_structure.K*codata.c/(self.Lorentz_factor())*1e-11
        atol_vz = ((self.magnetic_structure.K**2) * codata.c) / (4.*self.Lorentz_factor()**2)*1e-11
        atol_x = (atol_vx*self.magnetic_structure.period_length)/(codata.c*self.average_z_speed_in_undulator()*2.*np.pi)*1e-11
        atol_z = (atol_vz*self.magnetic_structure.period_length)/(codata.c*self.average_z_speed_in_undulator()*2.*np.pi)*1e-11
        return np.array([atol_vx,1e-10,atol_vz,atol_x,1e-10,atol_z])

    def rtol_for_ODE_method(self):
        return 1e-11

    ###########
    #property of the radiation
    ##################

    def angle_deflection_max(self):
        gamma=self.Lorentz_factor()
        return self.magnetic_structure.K/gamma

    # def angle_deflection_central_cone(self):
    #     # ring1=self.angle_ring_number(harmonic_number=1,ring_number=1)
    #     # return ring1/4.
    #     N=self.magnetic_structure.period_number()
    #     gamma=self.Lorentz_factor()
    #     return 1.5/(gamma*np.sqrt(N))

    def angle_deflection_central_cone(self,harmonic_number=1):
        # this is the angle of central cone sigma
        N=self.magnetic_structure.period_number()
        gamma=self.Lorentz_factor()
        return (1 / gamma) * np.sqrt( (1 + 0.5 * self.magnetic_structure.K**2) / (N * harmonic_number))


    def angle_ring_number(self,harmonic_number,ring_number):
        racine=(ring_number/harmonic_number)*(1.+(self.magnetic_structure.K**2)/2.)
        theta = np.sqrt(racine)/self.Lorentz_factor()
        return theta

    def harmonic_frequency(self,harmonic_number=1):
        gamma = self.Lorentz_factor()
        first_harm = ((2.0 * gamma ** 2) / (1.0 + (self.magnetic_structure.K ** 2) / 2.0)) * (
                (2.0 * np.pi * codata.c) / self.magnetic_structure.period_length)
        return harmonic_number*first_harm



    #######3
    #theoretical result
    ##############

    def Fn(self,n):
        if n%2==1 :
            K=self.magnetic_structure.K
            cst1=((n*K)/(1.+(K**2)/2.))**2
            cst2 = (n * K**2) / (4. + (2.*K ** 2))
            Fn=cst1*(jn(0.5*(n-1),cst2)-jn(0.5*(n+1),cst2))**2
        else :
            Fn=0.0
        return Fn

    def Qn(self,n):
        if n==0 :
            raise Exception(' the harmonic number can not be 0')
        K=self.magnetic_structure.K
        res=(1.+0.5*K**2)*self.Fn(n)/n
        return res

    #electron energy en Gev
    def critical_frequency(self,electron_energy):
        critical_energy=self.magnetic_field_strength()*(electron_energy/2.9)**2*5.59e3
        return critical_energy/codata.hbar
    #TODO TALMAN

    #TODO a changer , faire des exemples
    def describe_ring(self,distance,harmonic_number,ring_number,t=None):
        if ring_number==0 :
            X=np.array([0.0])
            Y=X
        else :
            if t is None :
                t=np.linspace(0.0,0.5*np.pi,101)
            n=harmonic_number
            theta=self.angle_ring_number(harmonic_number=n, ring_number=ring_number)
            if distance==None :
                R=theta
            else :
                R=distance*np.tan(theta)
            X=R*np.cos(t)
            Y=R*np.sin(t)
        return X,Y

    # in photon /sec /1% /mrad*mrad
    def theoretical_flux_on_axis(self,frequency):
        n=int(frequency/self.harmonic_frequency(1))
        if n==0:
            n=1
        if n%2==1 :
            # see X-ray data booklet pag 2.7
            cst=1.744e14*((self.period_number()*self.Electron_energy())**2)*self.I_current()
            result=cst*self.Fn(n)
        else :
            result=0.0
        return  result

    def theoretical_flux_on_axis_omega_scan(self,n,omega_array):
        if isinstance(n,(int,float)):
            n = [n]

        result = np.zeros_like(omega_array)
        for i in range(len(n)):
            cte = self.theoretical_flux_on_axis(n[i])
            arg = np.pi * self.period_number() * (omega_array / self.harmonic_frequency(1) - n[i])
            sinc = np.sin(arg) / arg
            result += cte*sinc**2
        return  result

    def theoretical_flux_integrated_central_cone(self,n):
        # see X-ray data booklet pag 2.8
        N=self.magnetic_structure.period_number()
        I=self.I_current()
        res=1.431e14*N*I*self.Qn(n)
        return res





def Example1_undulator(undulator):

    undulator.print_parameters()

    print( " velocity average in Z direction (c units)")
    print(undulator.average_z_speed_in_undulator())
    print(" first harmonic frequency (Hz)")
    print(undulator.harmonic_frequency(1))
    print(" angle (rad) of the first ring for the first harmonic ")
    print(undulator.angle_ring_number(1,1))

    print(' magnetic field intensity (T)')
    print(undulator.magnetic_field_strength())

    print(' Magnetic field intensity in (0,0,0)')
    Bx = undulator.magnetic_field.Bx(z=0.0, y=0.0, x=0.0)
    By = undulator.magnetic_field.By(z=0.0, y=0.0, x=0.0)
    Bz = undulator.magnetic_field.Bz(z=0.0, y=0.0, x=0.0)
    B = np.array([Bx, By, Bz])
    print(B)

    print('On axis peak intensity Fn(K)')
    print(undulator.Fn(1))




if __name__ == "__main__" :
    electron_beam_test = ElectronBeam(Electron_energy=1.3, I_current=1.0)
    undulator_test = MagneticStructureUndulatorPlane( K=1.87, period_length=0.035, length=0.035 * 14)
    source=SourceUndulatorPlane(electron_beam = electron_beam_test, undulator=undulator_test)

    Example1_undulator(source)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import numpy as np
import scipy.constants as codata
import scipy.special as special
from pySRU.Source import Source,PLANE_UNDULATOR,BENDING_MAGNET
from pySRU.ElectronBeam import ElectronBeam



#TODO
class SourceBendingMagnet(Source):
    def __init__(self, electron_beam, magnetic_structure, magnetic_field=None):
        super(self.__class__, self).__init__(electron_beam=electron_beam,
                                             magnetic_field=magnetic_field,
                                             magnetic_structure=magnetic_structure)


    def magnetic_field_strength(self):
        return self.magnetic_structure.Bo


    def copy(self):
        return SourceBendingMagnet(electron_beam=self.electron_beam.copy(),
                                    magnetic_field=self.magnetic_field.copy(),
                                    magnetic_structure=self.magnetic_structure)



    def arc_length(self):
        return self.magnetic_structure.horizontal_div* self.magnetic_structure.radius_curv

    def horizontal_divergence(self):
        return self.magnetic_structure.horizontal_div


    def radius_curvature(self,):
        return self.magnetic_structure.radius_curv



    #TODO faire le tri
    def critical_energy (self):
        Ec= 0.665 * (self.Electron_energy() ) ** 2 * self.magnetic_structure.Bo
        return Ec*1e-6

    def critical_frequency2(self):
        critical_energy=self.critical_energy()
        return critical_energy/codata.hbar

    def critical_frequency(self):
        gamma=self.Lorentz_factor()
        res=3.*(gamma**3)*codata.c/(2.*self.radius_curvature())
        return res

    def H2(self,y):
        K23=special.kv((2./3.),0.5*y)
        return (y*K23)**2


    def theoretical_flux_on_axis(self,n):
        #n=frequency/self.critical_frequency()
        #print('y= %.3f'%y)
        result= 1.327e13*((self.Electron_energy())**2
                          )*self.I_current()*self.H2(n)
        return result

    def radiation_theoric(self,omega,observation_angle):
        gamma=self.Lorentz_factor()
        X=gamma*observation_angle
        y=omega/self.critical_frequency()
        xi=y*0.5*np.sqrt((1.+X**2)**3)
        cst=(3.*codata.alpha*(gamma**2)*1e-3*1e-6*self.I_current()*y**2)/(codata.e*4.*np.pi**2)
        rad=((1.+X**2)**2)*((special.kv((2./3.),xi))**2+((X**2)/(1.+X**2))*(special.kv((2./3.),xi))**2)
        return rad*cst






    def choose_distance_automatic(self, alpha=2,photon_frequency=None):
        return self.magnetic_structure.length*10**(alpha)


    def choose_nb_pts_trajectory(self, alpha=0.01,photon_frequency=None):
        return self.magnetic_structure.length*4.*10**(alpha)


    def choose_initial_contidion_automatic(self):
        Zo=-self.magnetic_structure.length*1.5
        ic=np.array([0.0,0.0,self.electron_speed()*codata.c,0.0,0.0,Zo])
        return ic

    def choose_photon_frequency(self):
        return self.critical_frequency()*0.1


    def choose_angle_deflection_max(self):
        theta=5.0/self.Lorentz_factor()
        return theta


    def angle_deflection_central_cone(self):
        return 1.0/self.Lorentz_factor()


    def analytical_times_vector(self, Nb_pts):
        to=-self.magnetic_structure.length*0.5/(self.electron_speed()*codata.c)
        t1=to+self.arc_length()/(self.electron_speed()*codata.c)
        time=np.linspace(to,t1,Nb_pts)
        return time


    def construct_times_vector(self, initial_contition, Nb_pts):
    #TODO a changer ne marche pas ???
        electron_speed=(self.electron_speed() * codata.c)
        to = -self.magnetic_structure.length * 0.5 / electron_speed
        t1 = to + self.arc_length() / electron_speed
        time_start=initial_contition[5]/ electron_speed
        if time_start <to :
            time=np.linspace(time_start,-time_start,Nb_pts)
        else :
            delta_t=t1-time_start
            if delta_t >0.0 :
                time=np.linspace(time_start,time_start+2.*delta_t,Nb_pts)
            else :#TODO a tester
                time=np.linspace(time_start,2.*time_start,Nb_pts)
        return time

    def rtol_for_ODE_method(self):
        return 1e-11

    def atol_for_ODE_method(self):
        gamma=self.Lorentz_factor()
        atol_vx= self.electron_speed()*codata.c*1e-11
        atol_vz = self.electron_speed()*codata.c*1e-11
        atol_x = (2.*self.radius_curvature())/(3.*gamma**2)*1e-2
        atol_z = (2.*self.radius_curvature())/(3.*gamma**2)*1e-2
        return np.array([atol_vx,1e-10,atol_vz,atol_x,1e-10,atol_z])

# source parameter

    def print_parameters(self):
        super(self.__class__, self).print_parameters()
        print('Bending Magnet')
        print('    length : %.5f (m)'%self.magnetic_structure.length)
        print('    magnetic_field_strength : %.5f (T)'%self.magnetic_structure.Bo)
        print('    horizontal divergeance : %.5f (rad ?)' % self.horizontal_divergence())
        print('    radius curvature : %.3f (m)' % self.radius_curvature())
        print('    critical frequency : %f *1e20 (unite ?)' %(self.critical_frequency()/1e20))





if __name__ == "__main__" :
    pass







# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import unittest
import numpy as np
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator
from pySRU.ElectronBeam import ElectronBeam
from pySRU.TrajectoryFactory import TRAJECTORY_METHOD_ANALYTIC,TRAJECTORY_METHOD_ODE
from pySRU.RadiationFactory import RADIATION_METHOD_APPROX_FARFIELD,RADIATION_METHOD_NEAR_FIELD
from pySRU.Simulation import create_simulation

class MainTest(unittest.TestCase):

    def simul_undulator_near_to_farfield(self,magnetic_struc,electron_beam, method_traj):

        ref = create_simulation(magnetic_structure=magnetic_struc,electron_beam=electron_beam,
                                     traj_method=method_traj,distance=100)
        ref.calculate_on_central_cone()

        rad_max=(ref.radiation.max())

        sim_test=ref.copy()


        sim_test.change_radiation_method(RADIATION_METHOD_NEAR_FIELD)
        rad_err=ref.radiation.error_max(sim_test.radiation)
        self.assertLessEqual(np.abs(sim_test.radiation.max()-rad_max)/rad_max,5e-2)
        self.assertLessEqual(rad_err / rad_max, 5e-2)

    def simul_undulator_traj_method(self, magnetic_struc,electron_beam, method_rad):

        sim_test = create_simulation(magnetic_structure=magnetic_struc, electron_beam=electron_beam,
                                     rad_method=method_rad,distance=100)
        sim_test.calculate_on_central_cone()

        ref = sim_test.copy()
        rad_max = (sim_test.radiation.max())

        sim_test.change_trajectory_method(TRAJECTORY_METHOD_ODE)
        rad_err = ref.radiation.error_max(sim_test.radiation)
        traj_err = ref.trajectory.error_max(sim_test.trajectory)
        self.assertLessEqual(np.abs(sim_test.radiation.max() - rad_max) / rad_max, 5e-2,4)
        self.assertLessEqual(traj_err[1] / ref.trajectory.x.max(), 5e-3)
        self.assertLessEqual(traj_err[3] / ref.trajectory.z.max(), 5e-3)
        self.assertLessEqual(traj_err[4] / ref.trajectory.v_x.max(), 5e-3)
        self.assertLessEqual(traj_err[6] / ref.trajectory.v_z.max(), 5e-3)
        self.assertLessEqual(traj_err[7] / ref.trajectory.a_x.max(), 5e-3)
        self.assertLessEqual(traj_err[9] / ref.trajectory.a_z.max(), 5e-3)
        self.assertLessEqual(np.abs(sim_test.radiation.max()-rad_max)/rad_max,5e-2)
        self.assertLessEqual(rad_err / rad_max, 5e-2)



    def simul_undulator_theoric(self, magnetic_struc, electron_beam, method_rad, method_traj):

        sim_test = create_simulation(magnetic_structure=magnetic_struc, electron_beam=electron_beam,
                                     rad_method=method_rad, traj_method=method_traj,distance=100)
        rad_axis=sim_test.radiation.intensity[0,0]
        rad_theo=sim_test.source.theoretical_flux_on_axis(1)#sim_test.radiation_fact.omega)

        print("rad axis %e , theo %e" % (rad_axis, rad_theo))



    def test_main(self):
        beam_test = ElectronBeam(Electron_energy=1.3, I_current=1.0)
        beam_ESRF = ElectronBeam(Electron_energy=6.0, I_current=0.2)
        und_test = Undulator(K=1.87, period_length=0.035, length=0.035 * 14)
        ESRF18 = Undulator(K=1.68, period_length=0.018, length=2.0)

        ##ESRFBM = BM(E=6.0e9, Bo=0.8, div=5e-3, R=25.0, I=0.2)

        self.simul_undulator_theoric(magnetic_struc=und_test, electron_beam=beam_test,
                                              method_rad=RADIATION_METHOD_APPROX_FARFIELD,method_traj=TRAJECTORY_METHOD_ODE)
        print("Intensity ok")


        print(' undulator test ')
        self.simul_undulator_near_to_farfield(magnetic_struc=und_test,electron_beam=beam_test
                                              ,method_traj=TRAJECTORY_METHOD_ANALYTIC)
        print("TRAJECTORY_METHOD_ANALYTIC ok")
        self.simul_undulator_near_to_farfield(magnetic_struc=und_test, electron_beam=beam_test
                                              ,method_traj=TRAJECTORY_METHOD_ODE)
        print("TRAJECTORY_METHOD_ODE ok")

        self.simul_undulator_traj_method(magnetic_struc=und_test, electron_beam=beam_test
                                         , method_rad=RADIATION_METHOD_APPROX_FARFIELD)
        print('APPROX FARFIELD ok')

        self.simul_undulator_traj_method(magnetic_struc=und_test, electron_beam=beam_test
                                         , method_rad=RADIATION_METHOD_NEAR_FIELD)
        print('NEAR FIELD ok')
        print(' ')
        print('undulator ESRF18')
        self.simul_undulator_near_to_farfield(magnetic_struc=ESRF18, electron_beam=beam_ESRF
                                              ,method_traj=TRAJECTORY_METHOD_ANALYTIC)
        print("TRAJECTORY_METHOD_ANALYTIC ok")

        self.simul_undulator_near_to_farfield(magnetic_struc=ESRF18, electron_beam=beam_ESRF
                                              , method_traj=TRAJECTORY_METHOD_ODE)
        print("TRAJECTORY_METHOD_ODE ok")

    #TODO marche pas ?
        self.simul_undulator_traj_method(magnetic_struc=ESRF18, electron_beam=beam_ESRF
                                             , method_rad=RADIATION_METHOD_APPROX_FARFIELD)
        print('APPROX FARFIELD ok')

        self.simul_undulator_traj_method(magnetic_struc=ESRF18, electron_beam=beam_ESRF
                                         , method_rad=RADIATION_METHOD_NEAR_FIELD)
        print('NEAR FIELD ok')



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import numpy as np
import scipy.integrate as integrate
from abc import abstractmethod


class Radiation(object):

    def __init__(self, intensity, X, Y, distance):
        self.intensity=intensity
        self.X=X
        self.Y=Y
        self.distance=distance

    def copy(self):
        return Radiation(intensity=self.intensity.copy(), X=self.X.copy(), Y=self.Y.copy(), distance=self.distance)



    def plot(self,title="",label=""):
        import matplotlib.pyplot as plt
        from mpl_toolkits.mplot3d import Axes3D

        if self.distance == None:
            zlabel = "Flux (phot/s/0.1%bw/mrad2)"
            xlabel = 'X [rad]'
            ylabel = 'Y [rad]'
        else:
            zlabel = "Flux (phot/s/0.1%bw/mm2)"
            xlabel = 'X [m]'
            ylabel = 'Y [m]'

        if self.X is None or self.Y is None:
            raise Exception(" X and Y must be array for plotting")
        if self.X.shape != self.Y.shape:
            raise Exception(" X and Y must have the same shape")
        fig = plt.figure()
        if len(self.X.shape) ==2 :
            ax = Axes3D(fig)
            ax.plot_surface(self.X, self.Y, self.intensity, rstride=1, cstride=1,cmap='hot_r')
        else :
            ax = fig.gca(projection='3d')
            ax.plot(self.X, self.Y, self.intensity, label=label)
            ax.legend()
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        ax.set_zlabel(zlabel)

        plt.title(title)
        plt.show()


    def difference_with(self,radiation2):
        if not self.XY_are_similar_to(radiation2):
            raise Exception('X and Y must be the same for each radiation')
        error=np.abs(self.intensity - radiation2.intensity)
        res=self.copy()
        res.intensity=error
        return res

    def relativ_difference_with(self,radiation2):
        diff=self.difference_with(radiation2=radiation2)
        rad_max_ref=self.intensity.max()
        diff.intensity *= 1./rad_max_ref
        return diff

    def integration(self,is_quadrant=0,use_flux_per_mrad2_or_mm2=1):
        if (self.X is None or self.Y is None):
            raise Exception(" X and Y must be array for integration")
        if self.X.shape != self.Y.shape:
            raise Exception(" X and Y must have the same shape")

        if len(self.X.shape)==2 :
            X = np.linspace(self.X.min(), self.X.max(), self.X.shape[0])
            Y = np.linspace(self.Y.min(), self.Y.max(), self.Y.shape[1])

            res1=integrate.trapz(self.intensity, X)
            res = integrate.trapz(res1, Y)

            # res = integrate.simps(integrate.simps(self.intensity, X), Y)

            # res = self.intensity.sum() * (self.X[1,0] - self.X[0,0]) * (self.Y[0,1] - self.X[0,0]) # regular grid only
        else : # X and Y are 1d array
            if len(self.X) == 1:
                res = self.intensity[0]
            else: # choix arbitraire
                XY = np.zeros_like(self.X)
                for i in range(1, len(self.X)):
                    XY[i] = XY[i-1]+np.sqrt((self.X[i] - self.X[i-1]) * 2 + (self.Y[i] - self.Y[i-1]) ** 2)
                res = np.trapz(self.intensity, XY)

        # Note that the value of flux is in phot/s/0.1%bw/mrad2 (or .../mm2) and
        # our grid is in rad (or m), therefore we must account for this:
        if use_flux_per_mrad2_or_mm2:
            res *= 1e6

        # in case the calculation is for a quadrant, the integral is four times the calculated value
        if is_quadrant:
            res *= 4
        return res


    def XY_are_similar_to(self, rad2):
        if self.X.shape!= rad2.X.shape :
            return False
        elif self.Y.shape != rad2.Y.shape :
            return False
        else :
            if len(self.X.shape)==2 and len(rad2.X.shape)==2 :
                egality=np.zeros(self.X.shape[0],dtype=bool)
                for i in range(len(egality)):
                    egality[i]=all(self.X[i]==rad2.X[i])
                if not all(egality) :
                    return False
            else :
                 if not all(self.X==rad2.X):
                    return False
            if len(self.Y.shape)==2 and len(rad2.Y.shape)==2 :
                egality=np.zeros(self.X.shape[0],dtype=bool)
                for i in range(len(egality)):
                    egality[i]=all(self.Y[i]==rad2.Y[i])
                if not all(egality) :
                    return False
            else :
                 if not all(self.Y==rad2.Y):
                    return False
        return  True

    def max(self):
        res=(self.intensity).max()
        return res

    def error_max(self,radiation2):
        if self.intensity.shape != radiation2.intensity.shape :
            raise Exception('the two radiation must have the same shape')
        return (np.abs(self.intensity - radiation2.intensity)).max()

    def relativ_error(self,radiation2):
        if (self.max() ==0.0) :
           raise Exception("Problem : radiation max is null")
        res=self.error_max(radiation2)/self.max()
        return res

    def change_Nb_pts(self,Nb_pts):
        if len(self.X.shape)==2 or len(self.Y.shape)==2 :
            X = np.linspace((self.X.min()).min(), (self.X.max()).max(), Nb_pts)
            Y = np.linspace((self.Y.min()).min(), (self.Y.max()).max(), Nb_pts)
            X_ones = np.ones_like(X)
            Y_ones = np.ones_like(Y)
            self.X = np.outer(X,Y_ones)
            self.Y = np.outer(X_ones,Y)
        else :
            self.X = np.linspace(self.X.min(), self.X.max(), Nb_pts)
            self.Y= np.linspace(self.Y.min(), self.Y.max(), Nb_pts)


#TODO possible que pour l'undulateur
    def plot_ring(self, Nb_pts=201):
        import matplotlib.pyplot as plt
        from mpl_toolkits.mplot3d import Axes3D
        if self.X is None or self.Y is None:
            raise Exception(" X and Y must be grid or a list for plotting")
        fig = plt.figure()
        ax = fig.gca(projection='3d')
        X = np.array([self.X[0]])
        Y = np.array([self.Y[0]])
        intensity = np.array([self.intensity[0]])
        ax.plot(X, Y, intensity, '^', label='ring number 0')
        ring_number = 1
        while (ring_number * Nb_pts < len(self.X)):
            X = self.X[(ring_number - 1) * Nb_pts + 1:ring_number * Nb_pts + 1]
            Y = self.Y[(ring_number - 1) * Nb_pts + 1:ring_number * Nb_pts + 1]
            intensity = self.intensity[(ring_number - 1) * Nb_pts + 1:ring_number * Nb_pts + 1]
            ax.plot(X, Y, intensity, label='ring number %d' % ring_number)
            ring_number += 1
        ax.set_xlabel("X")
        ax.set_ylabel('Y')
        ax.set_zlabel("itensity")
        ax.legend()
        plt.show()


def Exemple_Grid():
    X = np.linspace(0.0, 0.005, 101)
    Y = np.linspace(0.0, 0.005, 101)
    # X_grid, Y_grid = np.meshgrid(X, Y)

    X_grid = np.outer(X,np.ones_like(Y))
    Y_grid = np.outer(np.ones_like(X),Y)

    intensity = (X_grid * 1e3) ** 2 * (Y_grid * 1e3) ** 2
    rad = Radiation(intensity=intensity, X=X_grid, Y=Y_grid, distance=100)
    print('maximum intensity radiation')
    print(rad.max())
    print(' integration of the intensity on the grid X,Y')
    print(rad.integration())
    print(' radiation intensity plot 1')
    rad.plot(title='from grid: plot 1')

    print(' ')
    print('create a second radiation on the same grid')
    rad2 = rad.copy()
    rad2.intensity = (X_grid * 1e3) ** 2 + (Y_grid * 1e3) ** 2
    print('maximum intensity radiation')
    print(rad2.max())
    print(' integration of the intensity on the grid X,Y')
    print(rad2.integration())
    print(' radiation intensity plot 2')
    rad2.plot(title='from grid: plot 2')

    print(' ')
    print('create a third radiation which is the different between the 2 radiation before')
    diff = rad.difference_with(rad2)
    print('maximum intensity radiation')
    print(diff.max())
    print(' integration of the intensity on the grid X,Y')
    print(diff.integration())
    print(' radiation intensity plot DIFFERENCE')
    diff.plot(title='from grid: DIFFERENCE 1 minus 2')



def Exemple_List():
    X = np.linspace(0.0, 5., 101)
    Y = np.linspace(2.5, 2.5, 101)
    distance = 100

    intensity = (X ) ** 2 * (Y ) ** 2
    rad = Radiation(intensity=intensity, X=X, Y=Y, distance=distance)
    print('maximum intensity radiation')
    print(rad.max())
    print(' integration of the intensity on X,Y')
    print(rad.integration())
    print(' radiation intensity plot')
    rad.plot(title='from list: plot 1')

    print(' ')
    print('create a second radiation on the same X,Y')
    rad2 = rad.copy()
    rad2.intensity = (X ) ** 2 + (Y) ** 2
    print('maximum intensity radiation')
    print(rad2.max())
    print(' integration of the intensity on X,Y')
    print(rad2.integration())
    print(' radiation intensity plot')
    rad2.plot(title= 'from list: plot 2')

    print(' ')
    print('create a third radiation which is the different between the 2 radiation before')
    diff = rad.difference_with(rad2)
    print('maximum intensity radiation')
    print(diff.max())
    print(' integration of the intensity on X,Y')
    print(diff.integration())
    print(' radiation intensity plot')
    diff.plot(title='from list: DIFFERENCE')

if __name__ == "__main__" :

    Exemple_Grid()
    Exemple_List()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import unittest
import numpy as np
import scipy.constants as codata
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator
from pySRU.ElectronBeam import ElectronBeam
from pySRU.SourceUndulatorPlane import SourceUndulatorPlane
from pySRU.TrajectoryFactory import TrajectoryFactory,TRAJECTORY_METHOD_ANALYTIC, TRAJECTORY_METHOD_ODE
from pySRU.RadiationFactory import RadiationFactory , RADIATION_METHOD_APPROX_FARFIELD,RADIATION_METHOD_NEAR_FIELD

class RadiationFactoryTest(unittest.TestCase):

    #TODO des print sont cache qql part
    def test_create_radiation_undulator(self):
        undulator_test = Undulator(K=1.87, period_length=0.035, length=0.035 * 14)
        electron_beam_test = ElectronBeam(Electron_energy=1.3, I_current=1.0)
        source_test=SourceUndulatorPlane(undulator=undulator_test, electron_beam=electron_beam_test)
        traj_fact=TrajectoryFactory(Nb_pts=1001, method=TRAJECTORY_METHOD_ANALYTIC)
        traj=traj_fact.create_from_source(source_test)

        rad_fact = RadiationFactory(photon_frequency=source_test.harmonic_frequency(1), method=RADIATION_METHOD_NEAR_FIELD)

        rad=rad_fact.create_for_one_relativistic_electron(trajectory=traj, source=source_test)
        self.assertFalse(rad.X is None)
        self.assertFalse(rad.Y is None)
        self.assertFalse(rad.distance is None)





        rad_fact.method=RADIATION_METHOD_APPROX_FARFIELD

        rad2=rad_fact.create_for_one_relativistic_electron(trajectory=traj, source=source_test)
        self.assertTrue(rad2.distance == None)


        rad2=rad_fact.create_for_one_relativistic_electron(trajectory=traj, source=source_test,distance=rad.distance)
        self.assertFalse(rad.distance == None)
        err=rad.difference_with(rad2)

        self.assertTrue(rad.XY_are_similar_to(rad2))
        self.assertTrue(rad.XY_are_similar_to(err))
        self.assertTrue(rad.distance == rad2.distance)
        self.assertTrue(err.distance == rad2.distance)
        self.assertGreaterEqual(err.intensity.min(),0.0)
        self.assertLessEqual(err.max(), rad.max()*1e-1) # srio changed 1e-3 by 1e-1


        traj_test2=TrajectoryFactory(Nb_pts=1001, method=TRAJECTORY_METHOD_ODE,
                                     initial_condition=traj_fact.initial_condition).create_from_source(source_test)

        rad3=rad_fact.create_for_one_relativistic_electron(trajectory=traj_test2,source=source_test,distance=rad.distance)
        err = rad2.difference_with(rad3)
        self.assertLessEqual(err.max(),rad2.max()*1e-3)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import numpy as np





class MagneticField(object):
    def __init__(self,Bx,By,Bz):
        self.Bx = Bx
        self.By = By
        self.Bz = Bz

    def copy(self):
        if type(self.Bx)==np.ndarray:
            Bx = self.Bx.copy()
        else :
            Bx=self.Bx
        if type(self.By)==np.ndarray:
            By = self.By.copy()
        else :
            By=self.By
        if type(self.Bz)==np.ndarray:
            Bz = self.Bz.copy()
        else :
            Bz=self.Bz
        return MagneticField(Bx=Bx, By=By, Bz=Bz)

    #faire une fct aui fait l'interpolation  et elargissement
    #

    #TODO a modifier
    # enelarge 2 vector for the interpolation of the magnetic field use in trajectory_undulator_from_magnetic_field2
    def enlargement_vector_for_interpolation(self,Z,Y ,nb_enlarg=0):
        dz = Z[1] -Z[0]
        enlarg_1 = np.linspace(Z[0] - nb_enlarg * dz, Z[0] - dz, nb_enlarg)
        enlarg_2 = np.linspace(Z[- 1] + dz, Z[- 1] + nb_enlarg * dz, nb_enlarg)
        Z = np.concatenate((enlarg_1, Z))
        Z = np.concatenate((Z, enlarg_2))
        if (type(Y )==np.ndarray) :
            dy = Y [1] - Y [0]
            enlarg_1 = np.linspace(Y [0] - nb_enlarg * dz, Y [0] - dy, nb_enlarg)
            enlarg_2 = np.linspace(Y [- 1] + dy, Y [- 1] + nb_enlarg * dy, nb_enlarg)
            Y  = np.concatenate((enlarg_1, Y ))
            Y = np.concatenate((Y , enlarg_2))
        if (type(self.By)==np.ndarray) :
            enlarg_3 = np.zeros(nb_enlarg)
            self.By = np.concatenate((enlarg_3, self.By))
            self.By = np.concatenate((self.By, enlarg_3))
        return Z,Y




    def plot_z(self,X,Y,Z,title="Magnetic field",field_component=None):
        import matplotlib.pyplot as plt

        if ( (field_component == None) or (field_component == 0) or (field_component == 'x') ):
            plt.plot(Z, self.Bx(z=Z,y=Y,x=X))

            plt.ylabel('Bx [T]')
            plt.xlabel('Z [m]')
            plt.title(title+" Bx = f(z) ")
            plt.show()

        if ( (field_component == None) or (field_component == 1) or (field_component == 'y') ):
            plt.plot(Z, self.By(z=Z,y=Y,x=X))
            plt.ylabel('By [T]')
            plt.xlabel('Z [m]')
            plt.title(title+" By = f(z) ")
            plt.show()

        if ( (field_component == None) or (field_component == 2) or (field_component == 'z') ):
            plt.plot(Z, self.Bz(z=Z,y=Y,x=X))
            plt.ylabel('Bz [T]')
            plt.xlabel('Z [m]')
            plt.title(title+" Bz = f(z) ")
            plt.show()

# # TODO A CHANGER COMME PLOT_z
        #TODO quiver plot ?
#     def plot_y(self):
#         plt.plot(self.y, self.Bx(self.z, self.y))
#         plt.title(" Bx = f(y) ")
#         plt.xlabel('Bx')
#         plt.ylabel('y')
#         plt.show()
#
#         plt.plot(self.y, self.By(self.z, self.y))
#         plt.title(" By = f(y) ")
#         plt.xlabel('By')
#         plt.ylabel('y')
#         plt.show()
#
#         plt.plot(self.y, self.Bz(self.z, self.y))
#         plt.title(" Bz = f(y) ")
#         plt.xlabel('Bz')
#         plt.ylabel('y')
#         plt.show()


if __name__ == "__main__" :
    Bx = lambda x,y,z : np.cos(y*z)
    By = lambda x,y,z : np.sin(x)*np.cos(z)
    Bz = lambda x,y,z : np.sin(y*z)

    B=MagneticField(Bx=Bx,By=By,Bz=Bz)

    Z=np.linspace(0.0,2.*np.pi,10000)
    Y=np.linspace(0.0,2.*np.pi,10000)
    X=np.linspace(0.0,2.*np.pi,10000)

    print('plot X=pi/2 , Y=1 , Z in [0,2 pi]')
    B.plot_z(X=np.pi*0.5,Y=1.0,Z=Z)
    print('plot X= pi/2 , Y and Z in [0,2 pi]')
    B.plot_z(X=np.pi*0.5,Y=Y,Z=Z)
    print('plot X , Y and Z in [0,2 pi]')
    B.plot_z(X=X,Y=Y,Z=Z)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import unittest
import numpy as np
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator


class MagneticStructureTest(unittest.TestCase):

    def test_copy_undulator(self):
        K = 1.87
        period_length = 0.035
        length = 0.035 * 14
        undulator = Undulator(K=K, period_length=period_length, length=length)
        undulator2 = undulator.copy()

        undulator2.K=1.5
        self.assertEqual(undulator.K,1.87)
        self.assertEqual(undulator2.K ,1.5)

        self.assertEqual(undulator.period_number(), 14)



    def test_create_magnetic_field(self):
        K = 1.87
        lambda_u = 0.035
        L = 0.035 * 12
        undulator = Undulator(K=K, period_length=lambda_u, length=L)
        Z=np.linspace(-1.,1.,101)
        Y=0.0
        X=0.0
        B=undulator.create_magnetic_field(harmonic_number=1)
        By=B.By(Z,Y,X)
        self.assertTrue(By.shape==((101,)))

        Y=Z
        By=B.By(Z,Y,X)
        self.assertEqual(By.shape , ((101,)))



#TODO faire Bending magnet



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import unittest
import numpy as np
import scipy.integrate as integrate
import scipy.constants as codata
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator
from pySRU.ElectronBeam import ElectronBeam
from pySRU.SourceBendingmagnet import BENDING_MAGNET as BM
from pySRU.TrajectoryFactory import TrajectoryFactory,TRAJECTORY_METHOD_ANALYTIC, TRAJECTORY_METHOD_ODE
from pySRU.RadiationFactory import RadiationFactory , RADIATION_METHOD_APPROX_FARFIELD,RADIATION_METHOD_NEAR_FIELD
from pySRU.Simulation import Simulation,create_simulation

class MagneticFieldTest(unittest.TestCase):



    def create_magn_field_undulator_test(self, magnetic_structure, electron_beam,method_traj):


        sim_test = create_simulation(magnetic_structure=magnetic_structure,electron_beam=electron_beam,
                                     traj_method=method_traj,rad_method=RADIATION_METHOD_APPROX_FARFIELD)
        print('create')
        source_test=sim_test.source
        Zo=sim_test.trajectory_fact.initial_condition[5]
        Bo=source_test.magnetic_field_strength()
        lambda_u=source_test.magnetic_structure.period_length

        self.assertTrue(Zo<0.)
        Zo_analitic=source_test.magnetic_structure.length*0.5
        Z_test = np.linspace(-Zo_analitic, Zo_analitic, sim_test.trajectory_fact.Nb_pts)
        diff_mf = np.abs(source_test.magnetic_field.By(z=Z_test, y=0.0, x=0.0) -
                         Bo * np.cos((2.0 * np.pi / lambda_u) * Z_test))
        self.assertTrue(all(diff_mf < np.abs(Bo) * 1e-3))

        print('integration 1')
        int1 = integrate.quad((lambda z: source_test.magnetic_field.By(z=z, y=0.0, x=0.0)),Zo,-Zo
                              ,limit=int(source_test.choose_nb_pts_trajectory(2)))
        self.assertAlmostEqual(int1[0], 0.0, 2)
        print('integration 2')#TODO marche pas donne tjr zeros meme qd c'est faux
        int2 = integrate.quad((lambda z: z*source_test.magnetic_field.By(z=z, y=0.0, x=0.0)),Zo,-Zo
                              ,limit=int(source_test.choose_nb_pts_trajectory(2)))
        print(int2[0])
        self.assertAlmostEqual(int2[0], 0.0)


    # doen't work ...
    # def create_magn_field_test_BM(self, BM, method_traj, formule=1):
    #     distance = 250.0
    #     Xmax = distance * 1e-3
    #     Ymax = distance * 1e-3
    #
    #     traj_test = TrajectoryFactory(Nb_pts=2000, method=method_traj)
    #     rad_test = RadiationFactory(method=RADIATION_METHOD_APPROX_FARFIELD, omega=BM.omega1(), Nb_pts=100,
    #                                 formula=formule)
    #     sim_test = create_simulation(parameter=BM, trajectory_fact=traj_test, radiation_fact=rad_test,
    #                                  distance=distance, X_max=Xmax, Y_max=Ymax)
    #     if method_traj == TRAJECTORY_METHOD_ANALYTIC:
    #         Zo = BM.Zo_analitic()
    #     else:
    #         Zo = BM.Zo_symetry()
    #     self.assertEqual(sim_test.magnetic_field.z[0], Zo)
    #     self.assertEqual(sim_test.magnetic_field.z[-1], -Zo)
    #     self.assertEqual(len(sim_test.magnetic_field.z), sim_test.trajectory_fact.Nb_pts)
    #     Z_test = np.linspace(BM.Zo_analitic(), -BM.Zo_analitic(), sim_test.trajectory_fact.Nb_pts)
    #     diff_mf = np.abs(sim_test.magnetic_field.By(z=Z_test, y=0.0, x=0.0) -
    #                      BM.Bo * np.cos((2.0 * np.pi / BM.lambda_u) * Z_test))
    #     self.assertTrue(all(diff_mf < np.abs(BM.Bo) * 1e-3))
    #     int1 = integrate.quad((lambda z: sim_test.magnetic_field.By(z=z, y=0.0, x=0.0)),
    #                           sim_test.magnetic_field.z[0],
    #                           sim_test.magnetic_field.z[-1])
    #     self.assertAlmostEqual(int1[0], 0.0, 5)
    #     int2 = integrate.quad((lambda z1: integrate.quad((lambda z: sim_test.magnetic_field.By(z=z, y=0.0, x=0.0)),
    #                                                      sim_test.magnetic_field.z[0], z1)[0])
    #                           , sim_test.magnetic_field.z[0], sim_test.magnetic_field.z[-1])
    #     self.assertAlmostEqual(int2[0], 0.0, 5)

    def test_magn_field(self):
        beam_test = ElectronBeam(Electron_energy=1.3, I_current=1.0)
        beam_ESRF = ElectronBeam(Electron_energy=6.0, I_current=0.2)
        und_test = Undulator(K=1.87, period_length=0.035, length=0.035 * 14)
        ESRF18 = Undulator(K=1.68, period_length=0.018, length=2.)

        self.create_magn_field_undulator_test(magnetic_structure=und_test,electron_beam=beam_test,
                                     method_traj=TRAJECTORY_METHOD_ODE)
        print("und_test, ok")

        self.create_magn_field_undulator_test(magnetic_structure=ESRF18,electron_beam=beam_ESRF,
                                    method_traj=TRAJECTORY_METHOD_ODE)
        print("esrf18, ok")

        # self.create_magn_field_test_BM(BM=ESRFBM, method_traj=TRAJECTORY_METHOD_ODE, formule=1)
        # print("esrf BM ok")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import numpy as np
import scipy.constants as codata


# container class
# parameter of the beam
class ElectronBeam(object):
    def __init__(self, Electron_energy, I_current):
        self.Electron_energy=Electron_energy
        self.I_current=I_current

    def copy(self):
        return ElectronBeam(Electron_energy=self.Electron_energy, I_current=self.I_current)

    def Lorentz_factor(self):
        return 1e9*self.Electron_energy / (codata.m_e *  codata.c**2 / codata.e)

    def electron_speed(self):
        gamma = self.Lorentz_factor()
        Beta = np.sqrt(1.0 - 1.0 / gamma ** 2)
        return Beta

    def print_parameters(self):
        print(' Electron Beam')
        print('    Electron energy (GeV): %.3f' %self.Electron_energy)
        print('    Intensity (A): %.3f' % self.I_current)
        print('    Electron speed : %.10f' %self.electron_speed())
        print('    Lorentz factor : %.7f' % self.Lorentz_factor())

if __name__ == "__main__" :
    K = 1.87
    E = 1.3
    I = 1.0
    beam=ElectronBeam(Electron_energy=E, I_current=I)

    beam.print_parameters()

    print('Lorentz factor: %f'%beam.Lorentz_factor())



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

#
# Memorandum: 
#
# Install from sources: 
#     git clone https://github.com/oasys-kit/pySRU
#     cd pySRU
#     python setup.py install
#
# Upload to pypi (when uploading, increment the version number):
#     python setup.py register (only once, not longer needed)
#     python setup.py sdist
#     python setup.py upload
#          
# Install from pypi:
#     pip install pySRU
#

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

from setuptools import setup, Command
import sys
import os



cmdclass = {}


# create doc with sphinx: python setup.py build_doc
try:
    import sphinx
    import sphinx.util.console
    sphinx.util.console.color_terminal = lambda: False
    from sphinx.setup_command import BuildDoc
except ImportError:
    sphinx = None
    class SphinxExpectedCommand(Command):
        user_options = []

        def initialize_options(self):
            pass

        def finalize_options(self):
            pass

        def run(self):
            raise RuntimeError(
                'Sphinx is required to build or test the documentation.\n'
                'Please install Sphinx (http://www.sphinx-doc.org).')

if sphinx is not None:
    class BuildDocCommand(BuildDoc):

        def run(self):
            # make sure the python path is pointing to the newly built
            # code so that the documentation is built on this and not a
            # previously installed version

            build = self.get_finalized_command('build')
            sys.path.insert(0, os.path.abspath(build.build_lib))

            # Build the Users Guide in HTML and TeX format
            for builder in ['html', 'latex']:
                self.builder = builder
                self.builder_target_dir = os.path.join(self.build_dir, builder)
                self.mkpath(self.builder_target_dir)
                BuildDoc.run(self)
            sys.path.pop(0)
else:
    BuildDocCommand = SphinxExpectedCommand

cmdclass['build_doc'] = BuildDocCommand


setup(name='pySRU',
      version='0.5.4',
      description='Python synchrotron undulator calcution',
      author='Sophie Thery, Mark Glass, Manuel Sanchez del Rio',
      author_email='srio@esrf.eu',
      url='https://github.com/SophieTh/und_Sophie_2016/',
      packages=['pySRU', 'pySRU.tests'],
      install_requires=[
                        'numpy',
                        'scipy'
                       ],
      cmdclass=cmdclass,
     )




# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["S Thery, M Glass, M Sanchez del Rio - ESRF ISDD Advanced Analysis and Modelling"]
__license__ = "MIT"
__date__ = "31/08/2016"

import unittest
import numpy as np
import scipy.constants as codata
from pySRU.MagneticStructureUndulatorPlane import MagneticStructureUndulatorPlane as Undulator
from pySRU.SourceUndulatorPlane import SourceUndulatorPlane
from pySRU.ElectronBeam import ElectronBeam
from pySRU.TrajectoryFactory import TrajectoryFactory,TRAJECTORY_METHOD_ANALYTIC,   TRAJECTORY_METHOD_ODE

class TrajectoryFactoryTest(unittest.TestCase):

    #TODO a completer
    def test_create_trajectory(self):

        # test le trajectoire ana lytic (des valeurs special
        # le Beta doit est constant pour certaine methode
        # les produi scalaire de l'acc avec la vitesse est ...
        # difference max entre deux trajectoire

        K = 1.87
        lambda_u = 0.035
        L = 0.035 * 14
        E=1.3
        I=1.0
        undulator = Undulator(K=K, period_length=lambda_u, length=L)
        beam=ElectronBeam(Electron_energy=E, I_current=I)
        source=SourceUndulatorPlane(undulator=undulator, electron_beam=beam)

        fact_test = TrajectoryFactory(Nb_pts=201,method=TRAJECTORY_METHOD_ODE)
        traj_test = fact_test.create_from_source(source=source)

        self.assertFalse(fact_test.initial_condition is None)

        self.assertTrue(all(fact_test.initial_condition==source.choose_initial_contidion_automatic()))
        self.assertTrue(fact_test.method==TRAJECTORY_METHOD_ODE)

        scalar_product=traj_test.v_x*traj_test.a_x + traj_test.a_z*traj_test.v_z

        self.assertAlmostEqual(np.abs(scalar_product).max(),0.0,3)







